<?php


namespace App\Http\Controllers\admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Blog;
use App\Models\Category;
use App\Models\Enquiry;

use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;

use Illuminate\Support\Str;

class BlogController extends Controller
{
    public function index()
    {
        $blogs = Blog::with('category')->get(); // Eager load category for each blog
        $user = Auth::guard('admin')->user();
        $enquries =Enquiry::all();

        foreach ($blogs as $blog) {
            if ($blog->category && $blog->category->status == 'inactive' && $blog->status == 'active') {
                // Automatically deactivate blog if its category is inactive
                $blog->status = 'inactive';
                $blog->save();
            }
        }
    
        return view('backend.admin.blog.index', compact('blogs', 'user','enquries'));
    }
    
    

    // Show the form for creating a new blog
    public function create()
{
    // Fetch active categories
    $categories = Category::where('status', 'active')->get();
    $user = Auth::guard('admin')->user(); // Fetch the logged-in admin user
    $enquries =Enquiry::all();

    return view('backend.admin.blog.create', compact('categories','user','enquries'));
}



public function store(Request $request)
{
    // Step 1: Validate the incoming request
    $request->validate([
        'title' => 'required|string|max:255',
        'author' => 'required|string|max:255',
        'category' => 'required|exists:categories,id', // Validate category ID
        'content' => 'required|string',
        'status' => 'required|in:active,inactive',
        'publish_date' => 'required|date',
        'images' => 'nullable|array',
        'images.*' => 'image|mimes:jpg,jpeg,png,gif,svg|max:2048', // Corrected max size
        'seo_title' => 'nullable|string|max:255',
        'seo_description' => 'nullable|string|max:255',
        'seo_keywords' => 'nullable|string|max:255',
        'faqs' => 'nullable|array', // Validate FAQs as an array
    ]);

       // Process FAQs
       $array = [];
       if ($request->has('faqs')) {
           $faqQuestion = array_column($request->faqs, 'question');
           $faqAnswer = array_column($request->faqs, 'answer');
           foreach ($faqQuestion as $key => $question) {
               $array[] = [
                   'question' => $question,
                   'answer' => $faqAnswer[$key] ?? '',
               ];
           }
       }


    // Step 3: Handle file uploads (images)
    $images = [];
    if ($request->hasFile('images')) {
        foreach ($request->file('images') as $image) {
            $images[] = $image->store('blog_images', 'public'); // Store in public disk
        }
    }

    // Step 4: Generate a unique slug
    $slug = \Illuminate\Support\Str::slug($request->title);
    $slugCount = Blog::where('slug', 'like', $slug . '%')->count();
    if ($slugCount > 0) {
        $slug .= '-' . ($slugCount + 1);
    }
    $blog['faqs'] = json_encode($array);
//  dd($blog['faqs']);

    // Step 5: Store the blog post in the database
    try {
        $blog = Blog::create([
            'title' => $request->title,
            'author' => $request->author,
            'category_id' => $request->category, // Store category_id
            'content' => $request->content,
            'slug' => $slug,
            'status' => $request->status,
            'publish_date' => $request->publish_date,
            'images' => json_encode($images), // Store as JSON string
            'seo_title' => $request->seo_title,
            'seo_description' => $request->seo_description,
            'seo_keywords' => $request->seo_keywords,
            'faqs' => $blog['faqs'], // Store FAQs as JSON string
        ]);


        // Step 6: Send email to subscribers (asynchronous or try-catch block)
        if ($blog->status == 'active') {
            $this->sendEmailToSubscribers($blog);
        }

        return redirect()->route('blogs.index')->with('success', 'Blog created successfully and emails sent!');
    } catch (\Exception $e) {
        return back()->withErrors(['error' => 'Failed to create the blog: ' . $e->getMessage()]);
    }
}







    
    

   

public function edit($id)
{
    $blog = Blog::findOrFail($id);
    $categories = Category::all();  // Get all categories
    $user = Auth::guard('admin')->user(); // Fetch the logged-in admin user
    $enquries =Enquiry::all();
    $blog->faqs = json_decode($blog->faqs, true) ?? [];


    // Pass category_id to the view
    return view('backend.admin.blog.edit', compact('blog', 'categories','user','enquries'));
}



public function update(Request $request, $id)
{
    $request->validate([
        'title' => 'required|string|max:255',
        'category' => 'required|exists:categories,id',
        'content' => 'required|string',
        'images' => 'nullable|array',
        'images.*' => 'image|mimes:jpg,jpeg,png,gif,svg|max:2048',
        'status' => 'required|in:active,inactive',
        'seo_title' => 'nullable|string|max:255',
        'seo_description' => 'nullable|string|max:255',
        'seo_keywords' => 'nullable|string|max:255',
        'faqs' => 'nullable|array', // Validate FAQs as an array
    ]);


    // Process FAQs
    $array = [];
    if ($request->has('faqs')) {
        $faqQuestion = array_column($request->faqs, 'question');
        $faqAnswer = array_column($request->faqs, 'answer');
        foreach ($faqQuestion as $key => $question) {
            $array[] = [
                'question' => $question,
                'answer' => $faqAnswer[$key] ?? '',
            ];
        }
    }

    // Find the blog post
    $blog = Blog::findOrFail($id);

    $blog['faqs'] = json_encode($array);



    // Update blog fields
    $blog->title = $request->title;
    $blog->category_id = $request->category;
    $blog->content = $request->content;
    $blog->seo_title = $request->seo_title;
    $blog->seo_description = $request->seo_description;
    $blog->seo_keywords = $request->seo_keywords;

    $blog->status = $request->status;

    // Handle removed images
    if ($request->has('removed_images')) {
        $removedImages = json_decode($request->removed_images, true);
        
        // Delete images from storage
        foreach ($removedImages as $image) {
            Storage::delete('public/' . $image);
        }

        // Remove the deleted images from the database field
        $existingImages = json_decode($blog->images, true);
        $updatedImages = array_diff($existingImages, $removedImages);
        $blog->images = json_encode($updatedImages);
    }

    // Handle new images
    if ($request->hasFile('images')) {
        $newImages = [];
        foreach ($request->file('images') as $image) {
            $imagePath = $image->store('blog_images', 'public');
            $newImages[] = $imagePath;
        }

        // Merge new images with existing ones
        if ($blog->images) {
            $existingImages = json_decode($blog->images, true);
            $blog->images = json_encode(array_merge($existingImages, $newImages)); // Add new images
        } else {
            $blog->images = json_encode($newImages); // If no existing images, store new ones
        }
    }

    // Save updated blog
    $blog->save();

    return redirect()->route('blogs.index')->with('success', 'Blog updated successfully!');
}





    // Delete a blog
    public function destroy($id)
    {
        $blog = Blog::findOrFail($id);

        // Delete associated images from storage (if any)
        $images = json_decode($blog->images);
        if ($images) {
            foreach ($images as $imagePath) {
                Storage::disk('public')->delete($imagePath); // Delete image from storage
            }
        }

        // Delete the blog record
        $blog->delete();

        return redirect()->route('blogs.index')->with('success', 'Blog deleted successfully!');
    }



    public function updateStatus(Request $request, $id)
    {
        $blog = Blog::findOrFail($id);
    
        // Check if the category of the blog is active
        if ($blog->category && $blog->category->status == 'inactive') {
            return back()->with('error', 'Please activate the category before activating the blog.');
        }
    
        // Toggle blog status
        $blog->status = $blog->status == 'active' ? 'inactive' : 'active';
        $blog->save();
    
        return back()->with('success', 'Blog status updated successfully.');
    }
    

    private function sendEmailToSubscribers($blog)
    {
        // Fetch all subscribers
        $subscribers = \App\Models\Subscription::all();
    
        // Send email to each subscriber
        foreach ($subscribers as $subscriber) {
            try {
                \Mail::to($subscriber->email)->send(new \App\Mail\NewBlogEmail($blog));
            } catch (\Exception $e) {
                // Log error if email sending fails
                \Log::error("Failed to send email to {$subscriber->email}: {$e->getMessage()}");
            }
        }
    }


    
}





